@@ -13,12 +13,7 @@ use Digest::MD5 qw(md5_hex);
 use IPC::Shareable;
 use RPi::Const qw(:all);
 
-our $VERSION = '2.3633_01';
-
-tie my %shared_pi_info, 'IPC::Shareable', {
-    key => 'rpiw',
-    create => 1,
-};
+our $VERSION = '2.3633_02';
 
 sub gpio_layout {
     return $_[0]->gpio_layout;
@@ -149,7 +144,7 @@ sub register_pin {
         mode        => $pin->mode,
         comment     => $comment //= '',
         operation   => 'register',
-        requester   => $self->uuid
+        requester   => $self->uuid,
     );
 }
 sub unregister_pin {
@@ -160,27 +155,53 @@ sub unregister_pin {
         requester   => $self->uuid
     );
 }
-sub cleanup{
+sub unregister_object {
     my ($self) = @_;
 
-    if ($ENV{PWM_IN_USE}){
-        WiringPi::API::pwm_set_mode(PWM_DEFAULT_MODE);
-        WiringPi::API::pwm_set_clock(PWM_DEFAULT_CLOCK);
-        WiringPi::API::pwm_set_range(PWM_DEFAULT_RANGE);
+    $self->meta_lock;
+    my $meta = $self->meta_fetch;
+
+    delete $meta->{objects}->{$self->uuid};
+    $meta->{object_count}--;
+    $meta->{object_count} = 0 if $meta->{object_count} < 0;
+
+    $self->meta_store($meta);
+    $self->meta_unlock;
+
+    $self->{clean} = 1;
+}
+sub cleanup {
+    my ($self) = @_;
+
+    return if $self->{clean};
+
+    $self->meta_lock;
+    my $meta = $self->meta_fetch;
+
+    if ($meta->{pwm}{in_use}){
+        WiringPi::API::pwmSetMode(PWM_DEFAULT_MODE);
+        WiringPi::API::pwmSetClock(PWM_DEFAULT_CLOCK);
+        WiringPi::API::pwmSetRange(PWM_DEFAULT_RANGE);
+        $meta->{pwm}->{in_use} = 0;
     }
 
-    for my $pin (keys %{ $shared_pi_info{pins} }){
-        if (exists $shared_pi_info{pins}->{$pin}{users}{$self->uuid}){
-            WiringPi::API::pin_mode_alt($pin, $shared_pi_info{pins}->{$pin}{alt});
-            WiringPi::API::write_pin($pin, $shared_pi_info{pins}->{$pin}{state});
-            if ($shared_pi_info{pins}->{$pin}{mode} < 4){
-                WiringPi::API::pin_mode($pin, $shared_pi_info{pins}->{$pin}{mode});
-            }
-            delete $shared_pi_info{pins}->{$pin};
+    for my $pin (keys %{ $meta->{pins} }){
+
+        if (exists $meta->{pins}->{$pin}{users}{$self->uuid}){
+            WiringPi::API::pinModeAlt($pin, $meta->{pins}->{$pin}{alt});
+            WiringPi::API::digitalWrite($pin, $meta->{pins}->{$pin}{state});
+            delete $meta->{pins}->{$pin};
         }
     }
 
-    delete $shared_pi_info{objects}->{$self->uuid};
+    delete $meta->{objects}->{$self->uuid};
+    $meta->{object_count}--;
+    $meta->{object_count} = 0 if $meta->{object_count} < 0;
+
+    $self->meta_store($meta);
+    $self->meta_unlock;
+
+    $self->{clean} = 1;
 }
 sub _pin_registration {
     # manages the registration duties for pins
@@ -189,8 +210,12 @@ sub _pin_registration {
 
     my $pin = $param{pin};
 
+    $self->meta_lock;
+    my $meta = $self->meta_fetch;
+
     if (! defined $pin){
-        my @registered_pins = keys %{ $shared_pi_info{pins} };
+        my @registered_pins = keys %{ $meta->{pins} };
+        $self->meta_unlock;
         return \@registered_pins;
     }
 
@@ -198,34 +223,45 @@ sub _pin_registration {
 
     if ($param{operation} eq 'unregister'){
 
-        if (! $shared_pi_info{pins}->{$pin_num}{users}{$param{requester}} eq $self->uuid){
+        if (! $meta->{pins}{$pin_num}{users}{$param{requester}} eq $self->uuid){
+            $self->meta_unlock;
             return;
         }
-        if (exists $shared_pi_info{pins}->{$pin_num}){
-            $pin->mode_alt($shared_pi_info{pins}->{$pin_num}{alt});
-            $pin->write($shared_pi_info{pins}->{$pin_num}{state});
-            $pin->mode($shared_pi_info{pins}->{$pin_num}{mode});
-            delete $shared_pi_info{pins}->{$pin_num};
+        if (exists $meta->{pins}{$pin_num}){
+            $pin->mode_alt($meta->{pins}{$pin_num}{alt});
+            $pin->write($meta->{pins}{$pin_num}{state});
+            $pin->mode($meta->{pins}{$pin_num}{mode});
+
+            delete $meta->{pins}{$pin_num};
+
+            $self->meta_store($meta);
+            $self->meta_unlock;
+
             return;
         }
     }
 
     if (! exists $param{state} && ! exists $param{alt}) {
+        $self->meta_unlock;
         croak "_pin_registration() requires both 'alt' and 'state' params\n";
     }
 
     if ($param{operation} eq 'register'){
-        if (exists $shared_pi_info{pins}->{$pin_num}){
+        if (exists $self->{meta}{pins}->{$pin_num}){
+            $self->meta_unlock;
             croak "pin $pin_num is already in use, can't continue...\n";
         }
-        $shared_pi_info{pins}->{$pin_num}{alt} = $param{alt};
-        $shared_pi_info{pins}->{$pin_num}{state} = $param{state};
-        $shared_pi_info{pins}->{$pin_num}{mode} = $param{mode};
-        $shared_pi_info{pins}->{$pin_num}{comment} = $pin->comment;
-        $shared_pi_info{pins}->{$pin_num}{users}{$param{requester}}++;
+        $meta->{pins}{$pin_num}{alt} = $param{alt};
+        $meta->{pins}{$pin_num}{state} = $param{state};
+        $meta->{pins}{$pin_num}{mode} = $param{mode};
+        $meta->{pins}{$pin_num}{comment} = $pin->comment;
+        $meta->{pins}{$pin_num}{users}{$param{requester}}++
     }
 
-    my @registered_pins = keys %{ $shared_pi_info{pins} };
+    my @registered_pins = keys %{ $self->{meta}{pins} };
+
+    $self->meta_store($meta);
+    $self->meta_unlock;
 
     return \@registered_pins;
 }
@@ -458,11 +494,24 @@ Parameters:
 
 Mandatory: An object instance of L<RPi::Pin> class.
 
+=head2 unregister_object
+
+Removes an object from the shared memory data store.
+
+B<NOTE>: This should only be used for testing purposes. It's simply a way to
+remove objects from the register while bypassing the entire C<cleanup()>
+routine.
+
 =head2 cleanup
 
 Resets all registered pins back to default settings as they were before your
 program started. It's important that this method be called in each application.
 
+=head2 tidy
+
+Performs a basic cleanup. This should only be used in situations when you know
+for fact that no pins or anything have been used in your script.
+
 =head1 ENVIRONMENT VARIABLES
 
 There are certain environment variables available to aid in testing on
