@@ -3,6 +3,8 @@ package RPi::WiringPi::Util;
 use strict;
 use warnings;
 
+use base 'Exporter';
+
 use parent 'WiringPi::API';
 use Carp qw(croak);
 use Data::Dumper;
@@ -10,12 +12,7 @@ use Digest::MD5 qw(md5_hex);
 use IPC::Shareable;
 use RPi::Const qw(:all);
 
-our $VERSION = '2.3633_01';
-
-tie my %shared_pi_info, 'IPC::Shareable', {
-    key => 'rpiw',
-    create => 1,
-};
+our $VERSION = '2.3633_02';
 
 sub checksum {
      return md5_hex(rand());
@@ -24,18 +21,6 @@ sub dump_signal_handlers {
     my ($self) = @_;
     print Dumper $self->_signal_handlers;
 }
-sub dump_metadata {
-    my ($self) = @_;
-    print Dumper $self->metadata;
-}
-sub dump_object {
-    my ($self) = @_;
-    print Dumper $self;
-}
-sub metadata {
-    my %meta = %shared_pi_info;
-    return \%meta;
-}
 sub pin_map {
     my ($self, $scheme) = @_;
 
@@ -74,9 +59,7 @@ sub uuid {
     my ($self) = @_;
     return $self->{uuid};
 }
-sub clean_shared {
-    %shared_pi_info = ();
-}
+
 sub _vim{1;};
 1;
 
@@ -99,24 +82,6 @@ used independently.
 Returns a randomly generated 32-byte hexidecimal MD5 checksum. We use this
 internally to generate a UUID for each Pi object.
 
-=head2 dump_metadata
-
-Used for troubleshooting/development, dumps the system's meta data within the
-shared memory storage using L<Data::Dumper>.
-
-=head2 dump_object
-
-Used for troubleshooting/development, dumps the object using L<Data::Dumper>.
-
-=head2 metadata
-
-During operation, we store several pieces of meta data of both the Pi object
-as well as operational status information in shared memory.
-
-Call this method to get a copy of this meta information.
-
-Return: Hash reference containing the meta data.
-
 =head2 pin_map($scheme)
 
 Returns a hash reference in the following format:
@@ -156,10 +121,6 @@ For C<'GPIO'> scheme:
 
 Returns the Pi object's 32-byte hexidecimal unique identifier.
 
-=head2 clean_shared
-
-Overwrites the shared memory storage area.
-
 =head2 signal_handlers
 
 Returns a hash reference of the currently set signal handlers.
